@@ -20,6 +20,7 @@ from redis.exceptions import (
     DataError,
     InvalidPipelineStack,
     InvalidResponse,
+    MaxConnectionsError,
     OutOfMemoryError,
     PubSubError,
     ReadOnlyError,
@@ -45,9 +46,7 @@ def int_or_str(value):
         return value
 
 
-# This is the version of redis-py that is being used
-# for building and installing the lib.
-__version__ = "6.1.0"
+__version__ = "6.4.0"
 VERSION = tuple(map(int_or_str, __version__.split(".")))
 
 
@@ -67,6 +66,7 @@ __all__ = [
     "default_backoff",
     "InvalidPipelineStack",
     "InvalidResponse",
+    "MaxConnectionsError",
     "OutOfMemoryError",
     "PubSubError",
     "ReadOnlyError",
