@@ -13,7 +13,6 @@ from redis.cache import (
     EvictionPolicyType,
     LRUPolicy,
 )
-from redis.utils import HIREDIS_AVAILABLE
 from tests.conftest import _get_client, skip_if_resp_version, skip_if_server_version_lt
 
 
@@ -40,7 +39,6 @@ def r(request):
         yield client
 
 
-@pytest.mark.skipif(HIREDIS_AVAILABLE, reason="PythonParser only")
 @pytest.mark.onlynoncluster
 @skip_if_resp_version(2)
 @skip_if_server_version_lt("7.4.0")
@@ -124,6 +122,10 @@ class TestCache:
         ]
         # change key in redis (cause invalidation)
         r2.set("foo", "barbar")
+
+        # Add a small delay to allow invalidation to be processed
+        time.sleep(0.1)
+
         # Retrieves a new value from server and cache it
         assert r.get("foo") in [b"barbar", "barbar"]
         # Make sure that new value was cached
@@ -325,7 +327,6 @@ class TestCache:
         assert cache.size == 0
 
 
-@pytest.mark.skipif(HIREDIS_AVAILABLE, reason="PythonParser only")
 @pytest.mark.onlycluster
 @skip_if_resp_version(2)
 @skip_if_server_version_lt("7.4.0")
@@ -568,7 +569,6 @@ class TestClusterCache:
         assert cache.size == 0
 
 
-@pytest.mark.skipif(HIREDIS_AVAILABLE, reason="PythonParser only")
 @pytest.mark.onlynoncluster
 @skip_if_resp_version(2)
 @skip_if_server_version_lt("7.4.0")
@@ -674,7 +674,6 @@ class TestSentinelCache:
         assert cache.size == 0
 
 
-@pytest.mark.skipif(HIREDIS_AVAILABLE, reason="PythonParser only")
 @pytest.mark.onlynoncluster
 @skip_if_resp_version(2)
 @skip_if_server_version_lt("7.4.0")
