@@ -2,7 +2,6 @@ import contextlib
 import multiprocessing
 
 import pytest
-
 import redis
 from redis.connection import Connection, ConnectionPool
 from redis.exceptions import ConnectionError
@@ -19,6 +18,16 @@ def exit_callback(callback, *args):
 
 
 class TestMultiprocessing:
+    # On macOS and newly non-macOS POSIX systems (since Python 3.14),
+    # the default method has been changed to forkserver.
+    # The code in this module does not work with it,
+    # hence the explicit change to 'fork'
+    # See https://github.com/python/cpython/issues/125714
+    if multiprocessing.get_start_method() in ["forkserver", "spawn"]:
+        _mp_context = multiprocessing.get_context(method="fork")
+    else:
+        _mp_context = multiprocessing.get_context()
+
     # Test connection sharing between forks.
     # See issue #1085 for details.
 
@@ -42,7 +51,7 @@ class TestMultiprocessing:
             assert conn.read_response() == b"PONG"
             conn.disconnect()
 
-        proc = multiprocessing.Process(target=target, args=(conn,))
+        proc = self._mp_context.Process(target=target, args=(conn,))
         proc.start()
         proc.join(3)
         assert proc.exitcode == 0
@@ -72,7 +81,7 @@ class TestMultiprocessing:
                 conn.send_command("ping")
 
         ev = multiprocessing.Event()
-        proc = multiprocessing.Process(target=target, args=(conn, ev))
+        proc = self._mp_context.Process(target=target, args=(conn, ev))
         proc.start()
 
         conn.disconnect()
@@ -81,6 +90,40 @@ class TestMultiprocessing:
         proc.join(3)
         assert proc.exitcode == 0
 
+    @pytest.mark.parametrize("max_connections", [2, None])
+    def test_release_parent_connection_from_pool_in_child_process(
+        self, max_connections, master_host
+    ):
+        """
+        A connection owned by a parent should not decrease the _created_connections
+        counter in child when released - when the child process starts to use the
+        pool it resets all the counters that have been set in the parent process.
+        """
+
+        pool = ConnectionPool.from_url(
+            f"redis://{master_host[0]}:{master_host[1]}",
+            max_connections=max_connections,
+        )
+
+        parent_conn = pool.get_connection()
+
+        def target(pool, parent_conn):
+            with exit_callback(pool.disconnect):
+                child_conn = pool.get_connection()
+                assert child_conn.pid != parent_conn.pid
+                pool.release(child_conn)
+                assert pool._created_connections == 1
+                assert child_conn in pool._available_connections
+                pool.release(parent_conn)
+                assert pool._created_connections == 1
+                assert child_conn in pool._available_connections
+                assert parent_conn not in pool._available_connections
+
+        proc = self._mp_context.Process(target=target, args=(pool, parent_conn))
+        proc.start()
+        proc.join(3)
+        assert proc.exitcode == 0
+
     @pytest.mark.parametrize("max_connections", [1, 2, None])
     def test_pool(self, max_connections, master_host):
         """
@@ -92,7 +135,7 @@ class TestMultiprocessing:
             max_connections=max_connections,
         )
 
-        conn = pool.get_connection("ping")
+        conn = pool.get_connection()
         main_conn_pid = conn.pid
         with exit_callback(pool.release, conn):
             conn.send_command("ping")
@@ -100,20 +143,20 @@ class TestMultiprocessing:
 
         def target(pool):
             with exit_callback(pool.disconnect):
-                conn = pool.get_connection("ping")
+                conn = pool.get_connection()
                 assert conn.pid != main_conn_pid
                 with exit_callback(pool.release, conn):
                     assert conn.send_command("ping") is None
                     assert conn.read_response() == b"PONG"
 
-        proc = multiprocessing.Process(target=target, args=(pool,))
+        proc = self._mp_context.Process(target=target, args=(pool,))
         proc.start()
         proc.join(3)
         assert proc.exitcode == 0
 
         # Check that connection is still alive after fork process has exited
         # and disconnected the connections in its pool
-        conn = pool.get_connection("ping")
+        conn = pool.get_connection()
         with exit_callback(pool.release, conn):
             assert conn.send_command("ping") is None
             assert conn.read_response() == b"PONG"
@@ -129,12 +172,12 @@ class TestMultiprocessing:
             max_connections=max_connections,
         )
 
-        conn = pool.get_connection("ping")
+        conn = pool.get_connection()
         assert conn.send_command("ping") is None
         assert conn.read_response() == b"PONG"
 
         def target(pool, disconnect_event):
-            conn = pool.get_connection("ping")
+            conn = pool.get_connection()
             with exit_callback(pool.release, conn):
                 assert conn.send_command("ping") is None
                 assert conn.read_response() == b"PONG"
@@ -144,7 +187,7 @@ class TestMultiprocessing:
 
         ev = multiprocessing.Event()
 
-        proc = multiprocessing.Process(target=target, args=(pool, ev))
+        proc = self._mp_context.Process(target=target, args=(pool, ev))
         proc.start()
 
         pool.disconnect()
@@ -160,7 +203,7 @@ class TestMultiprocessing:
             assert client.ping() is True
             del client
 
-        proc = multiprocessing.Process(target=target, args=(r,))
+        proc = self._mp_context.Process(target=target, args=(r,))
         proc.start()
         proc.join(3)
         assert proc.exitcode == 0
